////////////////////////////////////////////////////////////////////////
// Class:       PowerSpectrum
// Module Type: analyzer
// File:        PowerSpectrum_module.cc
//
// Generated at Sun Dec  2 12:23:06 2012 by Alessandro Razeto & Nicola Rossi using artmod
// from art v1_02_04.
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Event.h"
#include "art/Utilities/InputTag.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "artdaq/DAQdata/Fragments.hh"
#include "ds50daq/DAQ/V172xFragment.hh"

#include "fftw3.h"
#include "TRootCanvas.h"
#include "TCanvas.h"
#include "TGraph.h"
#include "TAxis.h"

#include <numeric>
#include <vector>
#include <functional>
#include <algorithm>
#include <iostream>

namespace ds50 {
  class PowerSpectrum : public art::EDAnalyzer {
    public:
      explicit PowerSpectrum (fhicl::ParameterSet const & p);
      virtual ~PowerSpectrum ();

      void analyze (art::Event const & e) override;
      void beginRun(art::Run const &) override;

    private:
      std::string board_type_;
      std::unique_ptr<TCanvas> canvas_;
      std::unique_ptr<TGraph> graph_;
      int lg_;
      int prescale_;
      size_t fft_n_;
      double fft_fs_, *fft_in_, *fft_out_;
      fftw_plan fft_plan_;
      std::vector<double> frequencies_, power_spectrum_;
      size_t power_spectrum_entries_;
  };
}

ds50::PowerSpectrum::PowerSpectrum (fhicl::ParameterSet const & ps): board_type_(ps.get<std::string> ("board_type", "V1720")), lg_(ps.get<int> ("lg")), prescale_(ps.get<int> ("prescale", 5)) {
}

ds50::PowerSpectrum::~PowerSpectrum() { }

namespace {
  size_t next_fftw_size (int dw) {
    const int bases[] = { 2, 3, 5, 7 };
    int min_d = 0, min_b = 0;
    for (int b: bases) {
      int k = pow (b, ceil (log (dw) /log (b))) - dw;
      if (b == 2 || k < min_d) {
	min_d = k;
	min_b = b;
      } 
    }
    return pow (min_b, ceil (log (dw) /log (min_b)));
  }
}

void ds50::PowerSpectrum::analyze (art::Event const & e) {
  art::Handle<artdaq::Fragments> v172x;
  e.getByLabel ("daq", board_type_, v172x);
  if (!v172x.isValid ())  {
    e.getByLabel ("unhuff" + board_type_, board_type_, v172x);
    if (!v172x.isValid ()) return;
  }

  for (size_t i = 0; i < v172x->size(); ++i) {
    const auto& frag((*v172x)[i]);

    if (frag.sequenceID () % prescale_) return;
    if (!frag.hasMetadata ()) continue;

    V172xFragment b(frag);
    if (!frequencies_.size ()) {
      if (!frag.hasMetadata ()) continue;
      const V172xFragment::metadata *md = frag.metadata<V172xFragment::metadata> ();

      fftw_cleanup ();
      fft_n_ = next_fftw_size (md->record_length);
      fft_fs_ = md->sample_rate_MHz * 1e6;
      fft_in_ = (double *)fftw_malloc(sizeof(double) * fft_n_);
      fft_out_ = (double *)fftw_malloc(sizeof(double) * fft_n_);
      fft_plan_ = fftw_plan_r2r_1d (fft_n_, fft_in_, fft_out_, FFTW_R2HC, FFTW_ESTIMATE);

      std::cerr << "DW = " << md->record_length << " N = " << fft_n_ << " FS = " << fft_fs_ << std::endl;

      power_spectrum_.resize (fft_n_ / 2 + 1);
      frequencies_.resize (fft_n_ / 2 + 1);
      std::iota (frequencies_.begin (), frequencies_.end (), 0);
      double fs_step = fft_fs_ / 2. / 1e6 / frequencies_.back ();
      std::for_each (frequencies_.begin(), frequencies_.end (), [&fs_step](Double_t & d) { d *= fs_step; });

      graph_ = std::unique_ptr<TGraph>(new TGraph (frequencies_.size ()));
      std::copy (frequencies_.begin (), frequencies_.end (), graph_->GetX ());
      graph_->GetYaxis()->SetRangeUser (-140, -110);
      power_spectrum_entries_ = 0;
    }

    for (int ch=0; ch<8; ++ch) {
      int lg = b.board_id () * 8 + ch;                        
      if (lg != lg_ || !b.channel_present (ch)) continue;
      size_t dw = std::distance (b.chDataBegin (ch), b.chDataEnd (ch));
      if (dw > fft_n_) continue;
      double sum = std::accumulate (b.chDataBegin (ch), b.chDataEnd (ch), 0.);
      for (size_t i = 0; i < dw; i++) {
	//static const double pi = 2 * acos (0);
	//fft_in_[i] = (1 - cos (2 * pi * i / (dw - 1))) / sqrt(1.5) * (b.chDataBegin (ch)[i] - sum / dw) * 0.488e-3; // Hanning
	fft_in_[i] = (b.chDataBegin (ch)[i] - sum / dw) * 0.488e-3;
      }
      std::fill (fft_in_ + dw, fft_in_ + fft_n_, 0);
      
      fftw_execute (fft_plan_);

      double mwHz_scale = 1 / (dw * fft_fs_ / 2 * 50e-3);
      if (power_spectrum_entries_ > power_spectrum_.size () / 10) power_spectrum_entries_ = 0;
      power_spectrum_[0] = (power_spectrum_[0] * power_spectrum_entries_ + fft_out_[0] * fft_out_[0] * mwHz_scale) / (power_spectrum_entries_ + 1);  // DC component
      for (size_t i = 1; i < (fft_n_ + 1) / 2; i++) 
	power_spectrum_[i] = (power_spectrum_[i] * power_spectrum_entries_ + 
	    (fft_out_[i] * fft_out_[i] + fft_out_[fft_n_ - i] * fft_out_[fft_n_ - i]) * 
	    mwHz_scale) / (power_spectrum_entries_ + 1);
      if (fft_n_ % 2 == 0) power_spectrum_[fft_n_ / 2] = (power_spectrum_[fft_n_ / 2] * power_spectrum_entries_ + fft_out_[fft_n_ / 2] * fft_out_[fft_n_ / 2] * mwHz_scale) / (power_spectrum_entries_ + 1);  // Nyquist freq

      if (!(power_spectrum_entries_ % prescale_)) {
	for (size_t i = 0; i < power_spectrum_.size (); i++) graph_->GetY ()[i] = 10 * log10 (power_spectrum_[i]);
	//for (size_t i = 0; i < power_spectrum_.size (); i++) graph_->GetY ()[i] = fft_in_[i];
	graph_->Draw ("AL");
	canvas_->Modified();
	canvas_->Update();
      }

      power_spectrum_entries_ ++;
    }
  }

}	

void ds50::PowerSpectrum::beginRun(art::Run const &) { 
  graph_ = 0;
  canvas_ = 0;
  frequencies_.clear ();

  canvas_ = std::unique_ptr<TCanvas>(new TCanvas(Form("ps%d",lg_)));
  canvas_->SetLogx ();
  ((TRootCanvas*)canvas_->GetCanvasImp ())->DontCallClose ();
}


DEFINE_ART_MODULE(ds50::PowerSpectrum)
