#!/bin/bash

source `which setPortNumbers.sh`

# this function expects a number of arguments:
#  1) the DAQ command to be sent
#  2) the run number (dummy for non-start commands)
#  3) the pulser trigger rate
#  4) the laser trigger rate
#  5) whether to enable the random trigger (0 or 1)
#  6) the acqusition gate width (usec)
#  7) the majority logic low threshold
#  8) the compression level for ADC data [0..2]
#  9) whether to run online monitoring
# 10) the data directory
# 11) the logfile name
# 12) whether to write data to disk [0,1]
# 13) the desired number of events in the run (stop command)
# 14) the desired length of the run (minutes, stop command)
# 15) the desired size of the data file (bytes, stop command)
function launch() {
  ebComp=$8
  agComp=$8
  if [[ "$8" == "2" ]]; then
    ebComp=1
    agComp=3
  elif [[ "$8" == "1" ]]; then
    ebComp=1
    agComp=0
  fi

  ds50MasterControl.rb -s -c $1 \
    --v1495 dsfr1,$frPort0,0,$3,$4,$5,$6,$7,0 \
    --v1720 dsfr1,$frPort1,1,0,$6 \
    --v1720 dsfr1,$frPort2,2,0,$6 \
    --v1720 dsfr1,$frPort3,3,0,$6 \
    --v1720 dsfr2,$frPort4,0,0,$6 \
    --v1720 dsfr2,$frPort5,1,0,$6 \
    --eb dseb1,$ebPort0,$ebComp \
    --eb dseb2,$ebPort1,$ebComp \
    --eb dseb4,$ebPort2,$ebComp \
    --eb dseb5,$ebPort3,$ebComp \
    --ag dsag,$agPort0,4,$agComp \
    --data-dir ${10} --online-monitoring $9 \
    --write-data ${12} --event-count ${13} \
    --run-duration ${14} --file-size ${15} \
    --run-number $2 2>&1 | tee -a ${11}
}

scriptName=`basename $0`
usage () {
    echo "
Usage: ${scriptName} [options] <command> [command options]
Where command is one of:
  init, start, pause, resume, stop,
  shutdown, start-system, restart, reinit, exit,
  fast-shutdown, fast-restart, fast-reinit, or fast-exit
Options:
  -h, --help: prints this usage message
  -p <pulser rate>: specifies the rate of the pulser trigger
  -l <laser rate>: specifies the rate of the laser trigger
  -r : enables the random trigger
  -g <gate width>: specifies the acquisition gate width in usec [default=400]
  -t <low threshold>: specifies the low threshold for the majority
      logic trigger [default=5]
  -c <compression level>: specifies the ADC data compression level
      0 = no compression
      1 = compression, both raw and compressed data kept
      2 = compression, only compressed data kept [default]
  -m <on|off>: specifies whether to run online monitoring [default=on]
  -o <data dir>: specifies the directory for data files [default=/data/test]
  -C <comment>: specify the run comment
  -T <type>: specify the run type
  -N <run number>: specifies the run number (optional)
  -D : disables the writing of data to disk
  -e <event count>: specifies the desired number of events in the run (stop command)
  -d <run duration>: specifies the desired length of the run (minutes, stop command)
  -s <file size>: specifies the desired file size (bytes, stop command)
Notes:
  The start command expects a run number to be specified (-N).
  The primary commands are the following:
   * init - initializes (configures) the DAQ processes
   * start - starts a run
   * pause - pauses the run
   * resume - resumes the run
   * stop - stops the run
  Additional commands include:
   * shutdown - stops the run (if one is going), resets the DAQ processes
       to their ground state (if needed), and stops the MPI program (DAQ processes)
   * start-system - starts the MPI program (the DAQ processes)
   * restart - this is the same as a shutdown followed by a start-system
   * reinit - this is the same as a shutdown followed by a start-system and an init
   * exit - this resets the DAQ processes to their ground state, stops the MPI
       program, and exits PMT.
  Expert-level commands:
   * fast-shutdown - stops the MPI program (all DAQ processes) no matter what
       state they are in. This could have bad consequences if a run is going!
   * fast-restart - this is the same as a fast-shutdown followed by a start-system
   * fast-reinit - this is the same as a fast-shutdown followed by a start-system
       and an init
   * fast-exit - this stops the MPI program, and exits PMT.
Examples: ${scriptName} -p 32768 init
          ${scriptName} -N 101 start
" >&2
}

# parse the command-line options
originalCommand="$0 $*"
pulserRate=""
laserRate=""
randomTrigger=""
gateWidth="400"
lowThreshold=5
compressionLevel=2
onmonEnable=on
diskWriting=1
dwChoiceSpecified=0
dataDir="/data/incoming"
runNumber=""
runComment=""
runType=""
eventCount=0
runDuration=0
fileSize=0
OPTIND=1
while getopts "hp:l:rg:c:N:C:T:o:t:m:Dn:d:s:-:" opt; do
    if [ "$opt" = "-" ]; then
        opt=$OPTARG
    fi
    case $opt in
        h | help)
            usage
            exit 1
            ;;
        p)
            pulserRate=${OPTARG}
            ;;
        l)
            laserRate=${OPTARG}
            ;;
        r)
            randomTrigger=1
            ;;
        g)
            gateWidth=${OPTARG}
            ;;
        t)
            lowThreshold=${OPTARG}
            ;;
        c)
            compressionLevel=${OPTARG}
            ;;
        N)
            runNumber=${OPTARG}
            ;;
        C)
            runComment=${OPTARG}
            ;;
        T)
            runType=${OPTARG}
            ;;
        m)
            onmonEnable=${OPTARG}
            ;;
        o)
            dataDir=${OPTARG}
            ;;
        D)
            diskWriting=0
            dwChoiceSpecified=1
            ;;
        n)
            eventCount=${OPTARG}
            ;;
        d)
            runDuration=${OPTARG}
            ;;
        s)
            fileSize=${OPTARG}
            ;;
        *)
            usage
            exit 1
            ;;
    esac
done
shift $(($OPTIND - 1))

# fetch the command to run
if [ $# -lt 1 ]; then
    usage
    exit
fi
command=$1
shift

# verify that the command is one that we expect
if [[ "$command" != "start-system" ]] && \
   [[ "$command" != "init" ]] && \
   [[ "$command" != "start" ]] && \
   [[ "$command" != "pause" ]] && \
   [[ "$command" != "resume" ]] && \
   [[ "$command" != "stop" ]] && \
   [[ "$command" != "shutdown" ]] && \
   [[ "$command" != "fast-shutdown" ]] && \
   [[ "$command" != "restart" ]] && \
   [[ "$command" != "fast-restart" ]] && \
   [[ "$command" != "reinit" ]] && \
   [[ "$command" != "fast-reinit" ]] && \
   [[ "$command" != "exit" ]] && \
   [[ "$command" != "fast-exit" ]]; then
    echo "Invalid command."
    usage
    exit
fi

export PGPASSWORD=ner

# save the diskWriting choice from the init command
if [[ "$command" == "init" ]]; then
    echo "$diskWriting" > ~/.ds50daq.diskWriting
fi

# fetch the diskWriting choice for the start and stop commsnds
if [[ "$dwChoiceSpecified" == "0" ]]; then
    if [[ "$command" == "start" ]] || [[ "$command" == "stop" ]]; then
        if [[ -e ~/.ds50daq.diskWriting ]]; then
            diskWriting=`cat ~/.ds50daq.diskWriting`
        fi
    fi
fi

# verify that the expected arguments were provided
if [[ "$command" == "start" ]]; then
  if [[ "$diskWriting" == "1" ]]; then 
    if [[ "$runComment" == "" ]] && [[ "$runType" == "" ]]; then
      echo ""
      echo "*** RunType and RunComment must be specified"
      usage
      exit
    fi
    end=`psql -A -c 'select endtime from run.daqruns_old_format order by runid desc limit 1' -h ds50daq.lngs.infn.it darkside50 run | grep ^2`
    if [[ "$end" == "" ]]; then
      echo "Previous run not stopped"
      exit
    fi
  fi
  if [[ "$runNumber" == "" ]]; then
    if [[ "$diskWriting" == "1" ]]; then 
      runNumber=`psql -A -c 'select MAX(runid)+1 from run.daqruns_old_format' -h ds50daq.lngs.infn.it darkside50 run | grep ^[0-9]`
    else
      echo "*** A run number needs to be specified."
      exit
    fi
  fi
fi

if [[ "$command" == "stop" ]]; then
  if [[ "$diskWriting" == "1" ]]; then 
    end=`psql -A -c 'select endtime from run.daqruns_old_format order by runid desc limit 1' -h ds50daq.lngs.infn.it darkside50 run | grep ^2`
    if [[ "$end" != "" ]]; then
      echo "Threre are not active run"
      exit
    fi
  fi
fi

# fill in values for options that weren't specified
if [[ "$pulserRate" == "" ]]; then
    pulserRate=0
fi
if [[ "$laserRate" == "" ]]; then
    laserRate=0
fi
if [[ "$randomTrigger" == "" ]]; then
    randomTrigger=0
fi
if [[ "$runNumber" == "" ]]; then
    runNumber=101
fi

# translate the onmon enable flag
if [[ "$onmonEnable" == "on" ]]; then
    onmonEnable=1
else
    onmonEnable=0
fi

if [[ "$command" == "stop" ]]; then
  if [[ "$diskWriting" == "1" ]]; then 
    psql -q -c "UPDATE run.daqruns_old_format SET endtime='NOW' where runid=(select MAX(runid) from run.daqruns_old_format);"  -h ds50daq.lngs.infn.it darkside50 run 
  fi
fi

# verify that we don't have both a number of events and a time limit
# for stopping a run
if [[ "$command" == "stop" ]]; then
    if [[ "$eventCount" != "0" ]] && [[ "$runDuration" != "0" ]]; then
        echo ""
        echo "*** Only one of event count or run duration may be specified."
        usage
        exit
    fi
fi

# build the logfile name
TIMESTAMP=`date '+%Y%m%d%H%M%S'`
logFile="/daqlogs/masterControl/dsMC-${TIMESTAMP}-${command}.log"
echo "${originalCommand}" > $logFile
echo ">>> ${originalCommand}"

# invoke the requested command
if [[ "$command" == "shutdown" ]]; then
    # first send a stop command to end the run (in case it is needed)
    launch "stop" $runNumber $pulserRate $laserRate $randomTrigger \
        $gateWidth $lowThreshold $compressionLevel $onmonEnable $dataDir \
        $logFile $diskWriting $eventCount $runDuration $fileSize
    # next send a shutdown command to move the processes to their ground state
    launch "shutdown" $runNumber $pulserRate $laserRate $randomTrigger \
        $gateWidth $lowThreshold $compressionLevel $onmonEnable $dataDir \
        $logFile $diskWriting $eventCount $runDuration $fileSize
    # lastly, stop the MPI program
    xmlrpc localhost:${pmtPort}/RPC2 pmt.stopSystem
elif [[ "$command" == "start-system" ]]; then
    xmlrpc localhost:${pmtPort}/RPC2 pmt.startSystem
elif [[ "$command" == "restart" ]]; then
    # first send a stop command to end the run (in case it is needed)
    launch "stop" $runNumber $pulserRate $laserRate $randomTrigger \
        $gateWidth $lowThreshold $compressionLevel $onmonEnable $dataDir \
        $logFile $diskWriting $eventCount $runDuration $fileSize
    # next send a shutdown command to move the processes to their ground state
    launch "shutdown" $runNumber $pulserRate $laserRate $randomTrigger \
        $gateWidth $lowThreshold $compressionLevel $onmonEnable $dataDir \
        $logFile $diskWriting $eventCount $runDuration $fileSize
    # stop the MPI program
    xmlrpc localhost:${pmtPort}/RPC2 pmt.stopSystem
    # start the MPI program
    xmlrpc localhost:${pmtPort}/RPC2 pmt.startSystem
elif [[ "$command" == "reinit" ]]; then
    # first send a stop command to end the run (in case it is needed)
    launch "stop" $runNumber $pulserRate $laserRate $randomTrigger \
        $gateWidth $lowThreshold $compressionLevel $onmonEnable $dataDir \
        $logFile $diskWriting $eventCount $runDuration $fileSize
    # next send a shutdown command to move the processes to their ground state
    launch "shutdown" $runNumber $pulserRate $laserRate $randomTrigger \
        $gateWidth $lowThreshold $compressionLevel $onmonEnable $dataDir \
        $logFile $diskWriting $eventCount $runDuration $fileSize
    # stop the MPI program
    xmlrpc localhost:${pmtPort}/RPC2 pmt.stopSystem
    # start the MPI program
    xmlrpc localhost:${pmtPort}/RPC2 pmt.startSystem
    # send the init command to re-initialize the system
    sleep 5
    launch "init" $runNumber $pulserRate $laserRate $randomTrigger \
        $gateWidth $lowThreshold $compressionLevel $onmonEnable $dataDir \
        $logFile $diskWriting $eventCount $runDuration $fileSize
elif [[ "$command" == "exit" ]]; then
    launch "shutdown" $runNumber $pulserRate $laserRate $randomTrigger \
        $gateWidth $lowThreshold $compressionLevel $onmonEnable $dataDir \
        $logFile $diskWriting $eventCount $runDuration $fileSize
    xmlrpc localhost:${pmtPort}/RPC2 pmt.stopSystem
    xmlrpc localhost:${pmtPort}/RPC2 pmt.exit

elif [[ "$command" == "fast-shutdown" ]]; then
    xmlrpc localhost:${pmtPort}/RPC2 pmt.stopSystem
elif [[ "$command" == "fast-restart" ]]; then
    xmlrpc localhost:${pmtPort}/RPC2 pmt.stopSystem
    xmlrpc localhost:${pmtPort}/RPC2 pmt.startSystem
elif [[ "$command" == "fast-reinit" ]]; then
    xmlrpc localhost:${pmtPort}/RPC2 pmt.stopSystem
    xmlrpc localhost:${pmtPort}/RPC2 pmt.startSystem
    sleep 5
    launch "init" $runNumber $pulserRate $laserRate $randomTrigger \
        $gateWidth $lowThreshold $compressionLevel $onmonEnable $dataDir \
        $logFile $diskWriting $eventCount $runDuration $fileSize
elif [[ "$command" == "fast-exit" ]]; then
    xmlrpc localhost:${pmtPort}/RPC2 pmt.stopSystem
    xmlrpc localhost:${pmtPort}/RPC2 pmt.exit

else
    launch $command $runNumber $pulserRate $laserRate $randomTrigger \
        $gateWidth $lowThreshold $compressionLevel $onmonEnable $dataDir \
        $logFile $diskWriting $eventCount $runDuration $fileSize
fi

if [[ "$command" == "start" ]]; then
  if [[ "$diskWriting" == "1" ]]; then 
    psql -q -c "insert INTO run.daqruns_old_format (runid, starttime, type, comment) VALUES ($runNumber, 'NOW', '$runType', '$runComment');"  -h ds50daq.lngs.infn.it darkside50 run
  fi 
  echo "Run $runNumber started"
  echo "Run $runNumber started" >> $logFile
fi
