#!/usr/bin/env ruby

require "test/unit"
require "xmlrpc/client"
require "open3"
require "tempfile"
require "socket"

require "pmt"

class TestPMT < Test::Unit::TestCase
  def setup
    @configFileHandle = Tempfile.new("pmt_config")
    hostname = Socket.gethostname
    @configFileHandle.write("pmt.rb %s 8000\n" % [hostname])
    @configFileHandle.write("pmt.rb %s 8001\n" % [hostname])
    @configFileHandle.write("pmt.rb %s 8002\n" % [hostname])
    @configFileHandle.rewind

    @pmtInstance = nil
    @pmtThread = nil
  end

  def teardown
    @configFileHandle.close!

    if @pmtInstance != nil
      @pmtInstance.stop
      sleep(5)
      @pmtInstance = nil
    end
  end

  def testCommandLine
    # Verify that PMT correctly parses arguments on the command line and that
    # the executables can be read back through the XMLRPC status method.  Test
    # that everything started up correctly.
    @pmtInstance = PMT.new(["-p", "8080", @configFileHandle.path])
    @pmtInstance.start

    sleep(1)
    server = XMLRPC::Client.new("localhost", "/", 8080)
    status = server.call("pmt.status")

    possibleOptions = ["8000", "8001", "8002"]
    assert_equal(status.count, 3)
    status.each { |statusHash|
      assert_equal(statusHash["host"], Socket.gethostname)
      assert_equal(statusHash["state"], "running")
      assert_equal(statusHash["program"], "pmt.rb")
      assert_equal(statusHash["exitcode"], -1)
      portIndex = possibleOptions.index(statusHash["options"])
      assert_not_equal(portIndex, nil)
      possibleOptions.delete_at(portIndex)
    }
    assert_equal(possibleOptions.count, 0)
  end

  def testXMLRPCAdditions
    # Verify that executables can be added through the XMLRPC interface.
    @pmtInstance = PMT.new(["-p", "8080"])
    @pmtInstance.start

    sleep(1)
    server = XMLRPC::Client.new("localhost", "/", 8080)
    server.call("pmt.addExecutables", [{"program" => "pmt.rb", "host" => Socket.gethostname, "port" => "8000"},
                                       {"program" => "pmt.rb", "host" => Socket.gethostname, "port" => "8001"},
                                       {"program" => "pmt.rb", "host" => Socket.gethostname, "port" => "8002"}])
    
    status = server.call("pmt.status")
    possibleOptions = ["8000", "8001", "8002"]
    assert_equal(status.count, 3)
    status.each { |statusHash|
      assert_equal(statusHash["host"], Socket.gethostname)
      assert_equal(statusHash["state"], "idle")
      assert_equal(statusHash["program"], "pmt.rb")
      assert_equal(statusHash["exitcode"], -1)
      portIndex = possibleOptions.index(statusHash["options"])
      assert_not_equal(portIndex, nil)
      possibleOptions.delete_at(portIndex)
    }
    assert_equal(possibleOptions.count, 0)
  end
    
  def testStartupShutdown
    # Verify that PMT can sucessfully use MPI to startup a handful of processes.
    # We'll do this by spwaning three copies of PMT and then make an XMLRPC call
    # out to one of them to verify they're running.  This code will also 
    # exercise the MPI stop code in PMT that is supposed to clean everything up.
    @pmtInstance = PMT.new(["-p", "8080", @configFileHandle.path])
    @pmtInstance.start

    sleep(1)
    server = XMLRPC::Client.new("localhost", "/", 8080)
    server.call("pmt.startSystem")

    sleep(3)
    status = server.call("pmt.status")
    possibleOptions = ["8000", "8001", "8002"]
    assert_equal(status.count, 3)
    status.each { |statusHash|
      assert_equal(statusHash["host"], Socket.gethostname)
      assert_equal(statusHash["state"], "running")
      assert_equal(statusHash["program"], "pmt.rb")
      assert_equal(statusHash["exitcode"], -1)
      portIndex = possibleOptions.index(statusHash["options"])
      assert_not_equal(portIndex, nil)
      possibleOptions.delete_at(portIndex)
    }
    assert_equal(possibleOptions.count, 0)

    server = XMLRPC::Client.new("localhost", "/", 8000)
    status = server.call("pmt.alive")
    assert_equal(status, true)
  end

  def testReturnCode
    # Verify that the return code returned from PMT is correct and represents
    # what was actually returned by the MPI application.
    return true
  end

  def testBadMPIStart
    # Verify that PMT can handle problems starting MPI in a reasonable fashion.
    return true
  end

  def testBadAppStart
    # Verify that the wrapper code handles problems starting up the given
    # application in a reasonable way.
    return true
  end
end
